<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

use sgae3\Controller\RouteController;

// App not installed?
if (!file_exists("./config/config.ini.php")) {
    header("Location: ./installer/install.phtml");
    die();
}

// Include global config: DB, etc.
include_once("./config/config.ini.php");
// Composer autoloader
require './vendor/autoload.php';
// Start session
session_start();
// Error handler
set_error_handler("sgae3\\Controller\\ErrorController::sgae3_error_handler");

$rC = new RouteController();

$rC->addAll("LoginController", "checkLogged");
$rC->addSome("admin", "LoginController", "checkIsAdmin");
$rC->addRoute("GET", "/login", "LoginController", "loginPage");
$rC->addRoute("POST", "/login", "LoginController", "loginPost");
$rC->addRoute("GET", "", "HomeController", "homePage");
$rC->addRoute("GET", "/", "HomeController", "homePage");
$rC->addRoute("GET", "/user", "HomeController", "homePage");
$rC->addRoute("POST", "/user/userGraphs", "GraphsController", "userGraphs");
$rC->addRoute("GET", "/queue", "HomeController", "queuePage");
$rC->addRoute("POST", "/queue/userQueueGraphs", "GraphsController", "userQueueGraphs");
$rC->addRoute("POST", "/queue/queueGraphs", "GraphsController", "queueGraphs");
$rC->addRoute("GET", "/cluster", "HomeController", "clusterPage");
$rC->addRoute("POST", "/cluster/userClusterGraphs", "GraphsController", "userClusterGraphs");
$rC->addRoute("POST", "/cluster/clusterGraphs", "GraphsController", "clustergraphs");
$rC->addRoute("GET", "/yearly", "HomeController", "yearlyPage");
$rC->addRoute("POST", "/yearly/yearlyGraphs", "GraphsController", "yearlyGraphs");
$rC->addRoute("GET", "/admin", "AdminController", "adminPage");
$rC->addRoute("POST", "/admin/queue/toogleEnabled", "AdminController", "toogleEnabledQueue");
$rC->addRoute("POST", "/admin/queue/new", "AdminController", "newQueue");
$rC->addRoute("POST", "/admin/queue/reset", "AdminController", "resetQueue");
$rC->addRoute("POST", "/admin/saveAdmins", "AdminController", "saveAdmins");
$rC->addRoute("POST", "/admin/saveQueueAdmins", "AdminController", "saveQueueAdmins");
$rC->addRoute("POST", "/admin/runCron", "AdminController", "runCron");
$rC->addRoute("POST", "/admin/getCronStatus", "AdminController", "getCronStatus");
$rC->addRoute("POST", "/admin/user/new", "AdminController", "newUser");
$rC->addRoute("POST", "/admin/user/delete", "AdminController", "deleteUser");
$rC->addRoute("POST", "/admin/user/changePassword", "AdminController", "changeUserPassword");
$rC->addRoute("GET", "/logout", "LoginController", "logout");

$rC->dispatch();
