<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Controller;

/*
 * Error controler
 * Log errors with "trigger_error ($error_msg, $error_type)"
 * Error messages will be written at <$sgae3_log>
 * Throw Exceptions as usual
 */
class ErrorController
{

    // PHP error handler
    function sgae3_error_handler($errno, $errstr, $errfile, $errline)
    {
        if (! (error_reporting() & $errno)) {
            // This error code is not included in error_reporting
            return false;
        }

        switch ($errno) {

            case E_USER_NOTICE:
                $text = "$errstr\n";
                break;
            case E_USER_ERROR:
                $text = "[sgae3_ERROR] $errstr in $errfile on line $errline";
                break;
            case E_USER_WARNING:
                $text = "[sgae3_WARNING] $errstr in $errfile on line $errline";
                break;
            case E_NOTICE:
                $text = "[NOTICE] $errstr in $errfile on line $errline";
                break;
            case E_WARNING:
                $text = "[WARNING] $errstr in $errfile on line $errline";
                break;
            case E_ERROR:
                $text = "[ERROR] PHP Error: $errstr in $errfile on line $errline";
                break;
            default:
                $text = "[UNKNOWN] $errstr in $errfile on line $errline";
                break;
        }

        $date = date("D d M H:i:s Y");
        if ($errno != E_USER_NOTICE) {
            $client = isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : " ";
            $referer = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : " ";
            $msg = "[$date] [client $client] $text, referer $referer \n";
        } else {
            $msg = "[$date] $text";
        }

        errorController::sgae3_error_log($msg);

        if ($errno == E_ERROR) {
            exit(1);
        }

        /* Don't execute PHP internal error handler */
        return true;
    }

    // Write to file
    function sgae3_error_log($error)
    {
        $sgae3_logfile = $GLOBALS['app_logfile'];
        if (! $fd = @fopen($sgae3_logfile, "ab")) {
            error_log("Unable to open '$sgae3_logfile'");
            return;
        }
        if (! @fwrite($fd, "$error")) {
            error_log("Unable to write to '$sgae3_logfile'");
            return;
        }
        fclose($fd);
    }
}