<?php
/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace sgae3\Service;

use Exception;
use sgae3\Repository\UserRepository;
use sgae3\Entity\User;

class userService
{

    public function findByUsername(string $username)
    {
        $userRepository = new UserRepository();
        return $userRepository->findByUsername($username);
    }

    public function find(int $user_id)
    {
        $userRepository = new UserRepository();
        return $userRepository->find($user_id);
    }

    public function newUser(string $username, string $password)
    {
        $userRepository = new UserRepository();
        $user = new User();
        $enc_password = password_hash($password, PASSWORD_DEFAULT);
        $user->create($username, $enc_password);
        try {
            $userRepository->save($user);
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function findUserByUsername(string $username)
    {
        $userRepository = new UserRepository();
        try {
            $user = $userRepository->findByUsername($username);
        } catch (Exception $e) {
            throw $e;
        }
        if (is_null($user)) {
            return false;
        }
        return $user;
    }

    public function getAllUsers(): array
    {
        $userRepository = new UserRepository();
        return $userRepository->findAll();
    }

    public function deleteUser(string $user_id): string
    {
        $userRepository = new UserRepository();

        try {
            $user = $userRepository->find($user_id);
            $username = $user->getUsername();
            if ($username == $_SESSION['username']) {
                throw new Exception("Please, be careful and do not delete yourself.");
            }
            $userRepository->delete($user);
        } catch (Exception $e) {
            throw $e;
        }

        return "User '$username' deleted";
    }


    public function changeUserPassword(string $user_id, string $password): string
    {
        $userRepository = new UserRepository();
        try {
            $user = $userRepository->find($user_id);
        } catch (Exception $e) {
            throw $e;
        }
        if (is_null($user)) {
            throw new Exception("User id '$user_id' does not exist");
        }
        $enc_password = password_hash($password, PASSWORD_DEFAULT);
        $user->setPassword($enc_password);
        try {
            $userRepository->update($user);
        } catch (Exception $e) {
            trigger_error($e->getMessage(), E_USER_ERROR);
            throw new Exception("Unable to update password");
        }
        return "Password changed";
    }
}
