/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Controls new render or update of charts
var user_rendered = false;
var cluster_rendered = false;

// cluster or user
var charts_var = "";
var charts_function = "";

// Charts config
const cluster_charts = [
    { id: "cluster_exec_time", data: "cluster_exec_time_data", func: "stacked_bar" },
    { id: "cluster_exec_jobs", data: "cluster_exec_jobs_data", func: "pie" },
    { id: "cluster_mem_usage", data: "cluster_mem_usage_data", func: "stacked_bar" },
    { id: "cluster_slots_usage", data: "cluster_slots_usage_data", func: "bar" },
    { id: "cluster_topten_node_usage", data: "cluster_topten_node_usage_data", func: "top_ten" },
    { id: "cluster_topten_users_usage", data: "cluster_topten_users_usage_data", func: "top_ten" },
    { id: "cluster_topten_memory_usage", data: "cluster_topten_memory_usage_data", func: "top_ten" },
    { id: "cluster_topten_execution_time", data: "cluster_topten_execution_time_data", func: "top_ten" },
    { id: "cluster_topten_execution_time_abs", data: "cluster_topten_execution_time_abs_data", func: "top_ten" }
];

// Also save charts in global objects for update
var cluster_exec_time, cluster_exec_jobs, cluster_mem_usage, cluster_slots_usage, cluster_topten_node_usage, cluster_topten_users_usage, cluster_topten_memory_usage,
    cluster_topten_execution_time, cluster_topten_execution_time_abs

$(function () {

    $("#nav_cluster").addClass("active");

    $("#cluster_graphs").hide();
    $("#user_graphs").hide();

    $("#date_range").change(function () {
        $("#start_time").prop('required', true);
        $("#end_time").prop('required', true);
    })
    $("#full_year").change(function () {
        $("#start_time").prop('required', false);
        $("#end_time").prop('required', false);
    })
    $("#submit").click(submit_form)
})

function submit_form() {

    // Delegated Browser/HTML5 validation
    if (!$("#year_select")[0].validity.valid) {
        display_error("Invalid year value");
        return false;
    }
    if (!$("#start_time")[0].validity.valid) {
        display_error("Invalid start time value");
        return false;
    }
    if (!$("#end_time")[0].validity.valid) {
        display_error("Invalid end time value");
        return false;
    }

    $.blockUI({
        message: '<h4>Loading data, please wait</h4><img src="./static/images/ajax-loader.gif"/>',
        css: {
            width: 'auto',
            left: '45%',
            padding: '15px',
        },
    });

    var mode
    var url
    if ($("#username").val() != "") {
        mode = "user";
        url = "cluster/userClusterGraphs";
        $("#user_graphs").show();
        $("#cluster_graphs").hide()
    } else {
        mode = "cluster";
        url = "cluster/clusterGraphs";
        $("#user_graphs").hide();
        $("#cluster_graphs").show()
    }

    // Ajax
    $.ajax({
        method: "POST",
        url: url,
        data: $("#user_form").serialize(),
        success: function (data) {
            try {
                var json = JSON.parse(data);
                if (json['status'] == "error") {
                    display_error(json['error'])
                    $.unblockUI();
                } else {
                    if (mode == "user") {
                        prepare_data_for_user_graphs(json['data'])
                        if (user_rendered) {
                            update_graphs(user_charts)
                        } else {
                            user_rendered = true
                            draw_graphs(user_charts)
                        }
                    } else if (mode == "cluster") {
                        prepare_data_for_cluster_graphs(json['data'])
                        if (cluster_rendered) {
                            update_graphs(cluster_charts)
                        } else {
                            cluster_rendered = true
                            draw_graphs(cluster_charts)
                        }
                    } else {
                        display_error("Unknown mode")
                    }
                }
                $.unblockUI();
            } catch (e) {
                display_error("An error has occurred")
                $.unblockUI();
                console.log(e)
            }
        },
        error: [ajax_error, $.unblockUI]
    });
    return false;
}

function prepare_data_for_cluster_graphs(json_data) {

    $("#cluster_running_jobs").text(json_data.cluster_running_jobs);
    $("#cluster_total_jobs").text(json_data.cluster_total_jobs);

    graph_footer("cluster_exec_time_footer",
        parseFloat(json_data.jobs_execution_time_longest / 60).toFixed(2) + " min",
        parseFloat(json_data.jobs_execution_time_shortest / 60).toFixed(2) + " min",
        parseFloat(json_data.jobs_execution_time_average / 60).toFixed(2) + " min");

    graph_footer("cluster_mem_usage_footer",
        parseFloat(json_data.jobs_memory_maximum / 1024).toFixed(2) + "MB",
        parseFloat(json_data.jobs_memory_minimum / 1024).toFixed(2) + "MB",
        parseFloat(json_data.jobs_memory_average / 1024).toFixed(2) + "MB");

    pie_graph_footer("cluster_exec_jobs_footer", json_data.jobs_failure, "failed", json_data.jobs_success, "success")

    cluster_exec_time_data.datasets[0].data[0] = json_data.jobs_execution_time_1d;
    cluster_exec_time_data.datasets[1].data[0] = json_data.jobs_execution_time_7d;
    cluster_exec_time_data.datasets[2].data[0] = json_data.jobs_execution_time_31d;
    cluster_exec_time_data.datasets[3].data[0] = json_data.jobs_execution_time_xxd;

    cluster_exec_jobs_data.datasets[0].data = [
        json_data.jobs_failure, json_data.jobs_success
    ]

    cluster_mem_usage_data.datasets[0].data[0] = json_data.jobs_memory_1gb;
    cluster_mem_usage_data.datasets[1].data[0] = json_data.jobs_memory_4gb;
    cluster_mem_usage_data.datasets[2].data[0] = json_data.jobs_memory_8gb;
    cluster_mem_usage_data.datasets[3].data[0] = json_data.jobs_memory_xxgb;

    cluster_slots_usage_data.datasets[0].data = [
        json_data.Slots_1,
        json_data.Slots_2,
        json_data.Slots_4,
        json_data.Slots_8,
        json_data.Slots_16,
        json_data.Slots_32,
        json_data.Slots_64,
        json_data.Slots_128,
        json_data.Slots_x
    ]

    cluster_topten_node_usage_data.labels = json_data.queue_topten_node_usage.labels;
    cluster_topten_node_usage_data.datasets[0].data = json_data.queue_topten_node_usage.values;

    cluster_topten_users_usage_data.labels = json_data.queue_topten_users_usage.labels;
    cluster_topten_users_usage_data.datasets[0].data = json_data.queue_topten_users_usage.values;

    cluster_topten_memory_usage_data.labels = json_data.queue_topten_mem_usage.labels;
    cluster_topten_memory_usage_data.datasets[0].data = json_data.queue_topten_mem_usage.values;

    cluster_topten_execution_time_data.labels = json_data.queue_topten_exec_time.labels;
    cluster_topten_execution_time_data.datasets[0].data = json_data.queue_topten_exec_time.values;

    cluster_topten_execution_time_abs_data.labels = json_data.queue_topten_exec_time_abs.labels;
    cluster_topten_execution_time_abs_data.datasets[0].data = json_data.queue_topten_exec_time_abs.values;

}

var cluster_exec_time_data = {
    labels: ['Number of jobs'],
    datasets: [{
        label: '< 1 day',
        data: [],
        backgroundColor: backgroundColor[0],
        borderColor: borderColor[0],
        borderWidth: 1
    }, {
        label: '1 day - 1 week',
        data: [],
        backgroundColor: backgroundColor[1],
        borderColor: borderColor[1],
        borderWidth: 1
    }, {
        label: '1 week - 1 month',
        data: [],
        backgroundColor: backgroundColor[2],
        borderColor: borderColor[2],
        borderWidth: 1
    }, {
        label: '> 1 month',
        data: [],
        backgroundColor: backgroundColor[3],
        borderColor: borderColor[3],
        borderWidth: 1
    }]
}

var cluster_exec_jobs_data = {
    labels: ['failed', 'completed'],
    datasets: [{
        label: 'Executed Jobs',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var cluster_mem_usage_data = {
    labels: ['Number of jobs'],
    datasets: [{
        label: '< 1GB',
        data: [],
        backgroundColor: backgroundColor[0],
        borderColor: borderColor[0],
        borderWidth: 1
    }, {
        label: '1GB - 4 GB',
        data: [],
        backgroundColor: backgroundColor[1],
        borderColor: borderColor[1],
        borderWidth: 1
    }, {
        label: '4GB - 8GB',
        data: [],
        backgroundColor: backgroundColor[2],
        borderColor: borderColor[2],
        borderWidth: 1
    }, {
        label: '> 8GB ',
        data: [],
        backgroundColor: backgroundColor[3],
        borderColor: borderColor[3],
        borderWidth: 1
    }]
}

var cluster_slots_usage_data = {
    labels: ['1', '2', '3-4', '5-8', '9-16', '17-32', '33-64', '65-128', '+128'],
    datasets: [{
        label: '# of Slots',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var cluster_topten_node_usage_data = {
    labels: [],
    datasets: [{
        label: '# of jobs',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var cluster_topten_users_usage_data = {
    labels: [],
    datasets: [{
        label: '# of jobs',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var cluster_topten_memory_usage_data = {
    labels: [],
    datasets: [{
        label: 'Avg. mem. (GB)',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var cluster_topten_execution_time_data = {
    labels: [],
    datasets: [{
        label: 'Avg. time. (hours)',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}

var cluster_topten_execution_time_abs_data = {
    labels: [],
    datasets: [{
        label: 'Max. time. (hours)',
        data: [],
        backgroundColor: backgroundColor,
        borderColor: borderColor,
        borderWidth: 1
    }]
}