/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

var borderColor = [
    'rgba(54, 162, 235, 1)', //blue
    'rgba(75, 192, 192, 1)', //green
    'rgba(227,26,28,1)', //red
    'rgba(255, 206, 86, 1)', //yellow
    'rgba(153, 102, 255, 1)', //purple
    'rgba(177, 89, 40, 1)', // Brown
    'rgba(255, 159, 64, 1)', // orange
    'rgba(50, 50, 50, 1)', // orange
    'rgba(255, 99, 132, 1)', //pink
    'rgba(255,255,153,1)', //LightYellow
    'rgba(202,178,214,1)', //LightPurple
]

var backgroundColor = [
    'rgba(54, 162, 235, 0.2)', //blue
    'rgba(75, 192, 192, 0.2)', //green
    'rgba(227,26,28,0.2)', //red
    'rgba(255, 206, 86, 0.2)', //yellow
    'rgba(153, 102, 255, 0.2)', //purple
    'rgba(177, 89, 40, 0.2)', // Brown
    'rgba(255, 159, 64, 0.2)', // orange
    'rgba(50, 50, 50, 0.2)', // orange
    'rgba(255, 99, 132, 0.2)', //pink
    'rgba(255, 255, 153, 0.2)', //LightYellow
    'rgba(202, 178, 214, 0.2)', //LightPurple
]

$(function () {
    $('#user_overlay').on('hide.bs.modal', function (e) {
        window['user_overlay_canvas'].destroy()
    })
    $('#queue_overlay').on('hide.bs.modal', function (e) {
        window['queue_overlay_canvas'].destroy()
    })
    $('#cluster_overlay').on('hide.bs.modal', function (e) {
        window['cluster_overlay_canvas'].destroy()
    })
    $('#yearly_overlay').on('hide.bs.modal', function (e) {
        window['yearly_overlay_canvas'].destroy()
    })
});

// Save overlay chart for destroying it on close.
var user_overlay_canvas = "";
var queue_overlay_canvas = "";

function draw_graphs(charts) {
    charts.forEach(element => {
        window[element.func](element.id, eval(element.data))
    })
}

function update_graphs(charts) {
    charts.forEach(element => {
        window[element.id].update()
    })
}

function graph_footer(element, max, min, avg) {
    $("#" + element).html(
        "<div class=\"row\">" +
        "<div class=\"col-4\">" + "<span class=\"small\">Max: " + numberWithCommas(max) + "</span></div>" +
        "<div class=\"col-4\">" + "<span class=\"small\">Min: " + numberWithCommas(min) + "</span></div>" +
        "<div class=\"col-4\">" + "<span class=\"small\">Avg: " + numberWithCommas(avg) + "</span></div>" +
        "</div>"
    )
    $("#" + element).css("padding-top", "0");
    $("#" + element).css("padding-bottom", "0")
    $("#" + element + " .row .col-4").css("padding-right", "0")
}

function pie_graph_footer(element, val1, label1, val2, label2) {
    $("#" + element).html(
        "<div class=\"row\">" +
        "<div class=\"col-5 offset-1\">" + "<span class=\"small\">" + label1 + ": " + numberWithCommas(val1) + "</span></div>" +
        "<div class=\"col-5\">" + "<span class=\"small\">" + label2 + ": " + numberWithCommas(val2) + "</span></div>" +
        "</div>"
    )
    $("#" + element).css("padding-top", "0");
    $("#" + element).css("padding-bottom", "0")
    $("#" + element + " .row .col-4").css("padding-right", "0")
}

function magnify(mode, id) {
    overlay = mode + "_overlay"
    header = overlay + "_header";
    body = overlay + "_body";
    canvas = overlay + "_canvas";
    charts_var = mode + "_charts";

    $('#' + header).text($("#" + id + "_title").html())
    found = eval(charts_var).find(element => element.id == id)
    $("#" + body).height(screen.height * 0.75)
    $('#' + overlay).modal('show')
    window[found.func](canvas, eval(found.data))
}

function numberWithCommas(x) {
    if (!x) return 0
    x = x.toString().replace(/\./, ",")
    return x
        .toString()
        .replace(/\B(?=(\d{3})+(?!\d))/g, ".");
};

function bar(container, data) {
    var ctx = $("#" + container);
    window[container] = new Chart(ctx, {
        type: 'bar',
        data: data,
        options: {
            tooltips: {
                callbacks: {
                    label: function (tooltipItem, data) {
                        return numberWithCommas(Number(tooltipItem.yLabel).toFixed(2));
                    }
                }
            },
            maintainAspectRatio: false,
            scales: {
                yAxes: [{
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    }
                }],
                xAxes: [{
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    }
                }]
            },
            legend: {
                position: 'bottom'
            },
            plugins: {
                labels: false
            }
        }
    });
}

function stacked_bar(container, data) {
    var ctx = $("#" + container);
    window[container] = new Chart(ctx, {
        type: 'bar',
        data: data,
        options: {
            tooltips: {
                callbacks: {
                    label: function (tooltipItem, data) {
                        return numberWithCommas(Number(tooltipItem.yLabel).toFixed(2));
                    }
                }
            },
            maintainAspectRatio: false,
            legend: {
                position: 'bottom',
                align: 'start'
            },
            scales: {
                xAxes: [{
                    stacked: true,
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    }
                }],
                yAxes: [{
                    stacked: true,
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    }
                }]
            },
            plugins: {
                labels: false
            }
        }
    });
}

function stacked_area(container, data) {
    var ctx = $("#" + container);
    window[container] = new Chart(ctx, {
        type: 'line',
        data: data,
        options: {
            tooltips: {
                callbacks: {
                    label: function (tooltipItem, data) {
                        return numberWithCommas(Number(tooltipItem.yLabel).toFixed(2));
                    }
                }
            },
            maintainAspectRatio: false,
            legend: {
                position: 'bottom',
                align: 'start'
            },
            scales: {
                xAxes: [{
                    stacked: true,
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    }
                }],
                yAxes: [{
                    stacked: true,
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    }
                }]
            },
            plugins: {
                labels: false
            }
        }
    });
}

function pie(container, data) {
    var ctx = $("#" + container);
    window[container] = new Chart(ctx, {
        type: 'pie',
        data: data,
        options: {
            tooltips: {
                callbacks: {
                    label: function (tooltipItem, data) {
                        return numberWithCommas(Number(data.datasets[0].data[tooltipItem.index]));
                    }
                }
            },
            maintainAspectRatio: false,
            legend: {
                position: 'bottom'
            },
            plugins: {
                labels: {
                    mode: 'percentage',
                    precision: 2
                }
            }
        }
    });
}

function top_ten(container, data) {
    var ctx = $("#" + container);
    window[container] = new Chart(ctx, {
        type: 'horizontalBar',
        data: data,
        options: {
            tooltips: {
                callbacks: {
                    label: function (tooltipItem, data) {
                        return numberWithCommas(Number(tooltipItem.value).toFixed(2));
                    },
                    title: function (tooltipItem, data) {
                        return;
                    }
                }
            },
            maintainAspectRatio: false,
            scales: {
                xAxes: [{
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    },
                    position: 'top'
                }],
                yAxes: [{
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        color: "rgba(0, 0, 0, 0)",
                    },
                }],
            },
            legend: {
                position: 'bottom'
            },
            plugins: {
                labels: false
            }
        }
    });
}