/*
S-GAE (Simple Graphical Accounting Engine) is a web application developed at /rdlab to 
graphically display the accounting information generated by our HPC queue manager.

Copyright (C) +2020  rdlab@cs.upc.edu - https://rdlab.cs.upc.edu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Controls new render or update of charts
var yearly_rendered = false;

// Charts config
const yearly_charts = [
    { id: "yearly_jobs_per_queue", data: "yearly_jobs_per_queue_data", func: "stacked_bar" },
    { id: "yearly_total_time_per_queue", data: "yearly_total_time_per_queue_data", func: "stacked_bar" },
    { id: "yearly_mem_per_queue", data: "yearly_mem_per_queue_data", func: "stacked_area" },
    { id: "yearly_time_per_queue", data: "yearly_time_per_queue_data", func: "stacked_area" },
    { id: "yearly_avg_wait_per_queue", data: "yearly_avg_wait_per_queue_data", func: "stacked_area" },

];

// Also save charts in global objects for update
var yearly_jobs_per_queue, yearly_total_time_per_queue, yearly_mem_per_queue, yearly_time_per_queue, yearly_avg_wait_per_queue

$(function () {
    $("#nav_yearly").addClass("active");
    submit_form()
})

function submit_form() {

    $.blockUI({
        message: '<h4>Loading data, please wait</h4><img src="./static/images/ajax-loader.gif"/>',
        css: {
            width: 'auto',
            left: '45%',
            padding: '15px',
        },
    });

    // Ajax
    $.ajax({
        method: "POST",
        url: "yearly/yearlyGraphs",
        success: function (data) {
            try {
                var json = JSON.parse(data);
                if (json['status'] == "error") {
                    display_error(json['error'])
                    $.unblockUI();
                } else {
                    prepare_data_for_yearly_graphs(json['data'])
                    if (yearly_rendered) {
                        update_graphs(yearly_charts)
                    } else {
                        yearly_rendered = true
                        draw_graphs(yearly_charts)
                    }
                }
                $.unblockUI();
            } catch (e) {
                display_error("An error has occurred")
                $.unblockUI();
                console.log(e)
            }
        },
        error: [ajax_error, $.unblockUI]
    });
    return false;
}

function prepare_data_for_yearly_graphs(json_data) {

    yearly_jobs_per_queue_data.labels = json_data.years
    yearly_total_time_per_queue_data.labels = json_data.years
    yearly_mem_per_queue_data.labels = json_data.years
    yearly_time_per_queue_data.labels = json_data.years
    yearly_avg_wait_per_queue_data.labels = json_data.years

    json_data.partitions.forEach((partition, index) => {
        
        //Dynamic Datasets
        yearly_jobs_per_queue_data.datasets[index] = {
            label: partition,
            data: Object.values(json_data.yearly_jobs_per_queue_data.values[partition]),
            backgroundColor: backgroundColor[index],
            borderColor: borderColor[index],
            borderWidth: 1
        }
        yearly_total_time_per_queue_data.datasets[index] = {
            label: partition,
            data: Object.values(json_data.yearly_jobs_execution_total_time_data.values[partition]),
            backgroundColor: backgroundColor[index],
            borderColor: borderColor[index],
            borderWidth: 1
        }
        yearly_mem_per_queue_data.datasets[index] = {
            label: partition,
            data: Object.values(json_data.yearly_avg_mem_per_queue_data.values[partition]),
            backgroundColor: backgroundColor[index],
            borderColor: borderColor[index],
            borderWidth: 1,
            fill: false
        }
        yearly_time_per_queue_data.datasets[index] = {
            label: partition,
            data: Object.values(json_data.yearly_avg_time_per_queue_data.values[partition]),
            backgroundColor: backgroundColor[index],
            borderColor: borderColor[index],
            borderWidth: 1,
            fill: false
        }
        yearly_avg_wait_per_queue_data.datasets[index] = {
            label: partition,
            data: Object.values(json_data.yearly_avg_wait_time_per_queue_data.values[partition]),
            backgroundColor: backgroundColor[index],
            borderColor: borderColor[index],
            borderWidth: 1,
            fill: false
        }
    });
}

yearly_jobs_per_queue_data = {
    labels: [],
    datasets: []
}

yearly_total_time_per_queue_data = {
    labels: [],
    datasets: []
}

yearly_mem_per_queue_data = {
    labels: [],
    datasets: []
}

yearly_time_per_queue_data = {
    labels: [],
    datasets: []
}

yearly_avg_wait_per_queue_data = {
    labels: [],
    datasets: []
}
